#!/usr/bin/env python
# Copyright (C) 2010 W. Trevor King <wking@drexel.edu>
#
# This file is part of SiteCorePy.
#
# SiteCorePy is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 2 of the License, or (at your
# option) any later version.
#
# SiteCorePy is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with SiteCorePy.  If not, see <http://www.gnu.org/licenses/>.

"""Move Professors from MySQL -> YAML.

Not really SiteCore related, but it seves as an example of what to do
before calling prof_import.
"""

import getpass
import logging
import re

import MySQLdb
import yaml

from .. import get_logger
from . import Name, Graduation, Contact, Bio, Professor


GRADUATION_REGEXP = re.compile('^(\S*) +(\S*), (\D*)(, *(\d*))?$')
"""Examples:

M.S.  Physics, University of Calcutta, Calcutta, India
Ph.D. Physics, University of Maryland, Maryland, 1967
"""

class SimpleDB (object):
    def __init__(self, verbose=0):
        self.db = None
        self.cursor = None
        self.table = None
        self.logger = get_logger(verbose)

    def connect(self, database, host='localhost',
                username=None, password=None):
        if username == None:
            username = getpass.getpass('username: ')
        if password == None:
            password = getpass.getpass('password: ')
        self.logger.info('logging in to %s:%s as %s'
                         % (host, database, username))
        self.db = MySQLdb.connect(host=host, user=username,
                                  passwd=password, db=database)
        del(username)
        del(password)
        self.cursor = self.db.cursor()

    def disconnect(self):
        self.logger.info('disconnecting')
        self.cursor = None
        self.db.close()

    def execute(self, command):
        self.logger.info(command)
        self.cursor.execute(command)

    def set_default_table(self, table):
        self.table = table

    def add_entry(self, dict_, table=None):
        if table == None:
            table = self.table
        dict_ = self._clean_dict(dict_)
        keys = dict_.keys()
        values = ["'%s'" % dict_[key] for key in keys]
        self.execute("INSERT INTO %s (%s) VALUES (%s)"
                     % (table, ', '.join(keys), ', '.join(values)))
        # error checking

    def list_entries(self, dict_=None, table=None):
        if table == None:
            table = self.table
        where = self._where_string(dict_)
        self.execute('SELECT * FROM %s%s' % (table, where))
        numrows = int(self.cursor.rowcount)
        fields = [dtuple[0] for dtuple in self.cursor.description]
        ret = []
        for x in range(0,numrows):
            row = self.cursor.fetchone()
            ret.append(dict(zip(fields, row)))
        return ret

    def delete_entries(self, dict_, table=None):
        if table == None:
            table = self.table
        where = self._where_string(dict_)
        self.execute('DELETE FROM %s%s' % (table, where))

    def _clean_dict(self, dict_=None):
        if dict_ == None:
            return None
        for key in dict_.keys():
            value = dict_.pop(key)
            value = MySQLdb.escape_string(value)
            key = MySQLdb.escape_string(key)
            assert key not in dict_, '%s in %s!' % (key, dict_)
            dict_[key] = value
        return dict_

    def _where_string(self, dict_=None):
        dict_ = self._clean_dict(dict_)
        if dict_ == None or len(dict_) == 0:
            where = ''
        else:
            where_args = []
            for key,value in dict_.items():
                where_args.append("%s='%s'" % (key,value))
            where = ' where %s' % ' and '.join(where_args)
        return where


def get_profs(db):
    """
mysql> describe people;
+--------------+--------------+------+-----+---------+----------------+
| Field        | Type         | Null | Key | Default | Extra          |
+--------------+--------------+------+-----+---------+----------------+
| ID           | int(11)      | NO   | PRI | NULL    | auto_increment | 
| category     | varchar(255) | YES  |     | NULL    |                | 
| lastname     | varchar(255) | YES  |     | NULL    |                | 
| firstname    | varchar(255) | YES  |     | NULL    |                | 
| email        | varchar(255) | YES  |     | NULL    |                | 
| office       | varchar(255) | YES  |     | NULL    |                | 
| phone        | varchar(255) | YES  |     | NULL    |                | 
| lab          | varchar(255) | YES  |     | NULL    |                | 
| labphone     | varchar(255) | YES  |     | NULL    |                | 
| grouppage    | varchar(255) | YES  |     | NULL    |                | 
| personalpage | varchar(255) | YES  |     | NULL    |                | 
| position     | varchar(255) | YES  |     | NULL    |                | 
| position1    | varchar(255) | YES  |     | NULL    |                | 
| degrees      | varchar(255) | YES  |     | NULL    |                | 
| research     | mediumtext   | YES  |     | NULL    |                | 
| profile      | mediumtext   | YES  |     | NULL    |                | 
+--------------+--------------+------+-----+---------+----------------+
16 rows in set (0.00 sec)

Where the relevant categories are
| Faculty            | 
| Research Faculty   | 
| Teaching Faculty   | 
| Professor Emeritus | 
| Adjunct Professor  | 
    """
    db.set_default_table('people')
    for category in ['Faculty',
                     'Research Faculty',
                     'Teaching Faculty',
                     'Professor Emeritus',
                     'Adjunct Professor']:
        for prof in db.list_entries({'category':category}):
            p = Professor()
            p.name = Name(
                first_middle=prof['firstname'],
                last=prof['lastname'],
                )
            p.title = prof['position']
            if prof['position1'] != None:
                p.title += ', %s' % prof['position1']
            p.graduations = []
            for degree in prof['degrees'].splitlines():
                m = GRADUATION_REGEXP.match(degree.strip())
                assert m != None, 'Misformed graduation: %s' % degree
                title,field,college,comma_year,year = m.groups()
                if year == '':
                    year = None
                p.graduations.append(
                    Graduation(
                        college=college,
                        field=field,
                        title=title,
                        year=year,
                        )
                    )
            p.contact = Contact(
                office=prof['office'],
                email=prof['email'],
                website=prof['personalpage'],
                phone=prof['phone'],
                lab=prof['lab'],
                lab_phone=prof['labphone'],
                )
            p.bio = Bio(
                specialization=prof['research'], # prof['grouppage']
                publications=None,
                profile=prof['profile'],
                cv=None,
                )
            db.logger.debug(unicode(p))
            yield p


def main(argv):
    import optparse
    usage = """%prog [options] PROF_FILE

Where PROF_FILE is the output YAML file receiving professor data.

Example:
  ./sc.py prof-export_mysql profs.yaml
"""
    p = optparse.OptionParser(usage)
    p.add_option('-d', '--database', metavar='NAME', dest='database',
                 help='Name of the MySQL database (%default)',
                 default='directory')
    p.add_option('-v', '--verbose', dest='verbose', action='count',
                 help='increment verbosity (%default)',
                 default=0)

    options,args = p.parse_args(argv)
    prof_file = args[0]

    db = SimpleDB(verbose=options.verbose)
    db.connect(database=options.database)
    try:
        profs = list(get_profs(db))
    finally:
        db.disconnect()

    yaml.save(db, profs)
